<?php
 /**
 * About module
 * http://www.aa-team.com
 * ======================
 *
 * @package		WooZoneVCSearch
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 */
! defined( 'ABSPATH' ) and exit;

if(class_exists('WooZoneVCSearch_dashboard') != true) {
	class WooZoneVCSearch_dashboard {
		
		const WXR_VERSION = '1.0';
		
		/**
		 * parent storage
		 *
		 * @var array
		 */
		public $parent = array();
		
		private $action = '';
		private $wp_import = null;
		
		/**
		 * The constructor
		 */
		public function __construct( $parent=array() )
		{
			// load parent
			$this->parent = $parent;
		}

		public function get_popular_cast()
		{
			$cast = array();
			$cast_results = get_terms( 'cast', array(
			    'orderby'    	=> 'count',
			    'order' 		=> 'DESC',
			    'number'		=> 12,
			    'hide_empty' 	=> 0
			) );

			if( $cast_results && count($cast_results) > 0 ){
				foreach ($cast_results as $key => $value) {

					$image_ID = get_term_meta( $value->term_id, '_thumbnail', true );
	
					$img = wp_get_attachment_image_src( $image_ID, array(200, 300) );
					$cast[$value->term_id] = array(
						'name' => $value->name,
						'count' => $value->count,
						'thumb' => ( trim($image_ID) != "" ? $img[0] : '' ),
					);
				}
			}

			return $cast;
		}

		public function get_popular_products()
		{
			$products = array();
			$args = array(
				'post_type' => 'movies',
				'meta_key' => '_WooZoneVCSearch_movie_hits',
				'orderby' => 'meta_value_num',
				'order' => 'DESC',
				'posts_per_page' => 12
			);
			$query = new WP_Query($args);
			
			if( (int) $query->found_posts > 0 ){
				foreach ( $query->posts as $post ) {

					$post_thumbnail_id = get_post_thumbnail_id( $post->ID );
					$poster_thumbnail = wp_get_attachment_image_src( $post_thumbnail_id, 'thumbnail' );
					$poster_thumbnail = $poster_thumbnail[0];
					$products[$post->ID] = array(
						'hits' => (int) get_post_meta( $post->ID, '_WooZoneVCSearch_movie_hits', true ),
						'poster_thumbnail' => $poster_thumbnail
					);
				}
			}

			return $products;
		}
		
		public function print_interface()
		{
			$html = array();
			$plugin_register = (string) get_option( "_" . WooZoneVCSearch()->alias . "_register_html" );

			$settings = get_option( 'WooZoneVCSearch_Plugin_Settings_' . WooZoneVCSearch()->get_campaign_id(), true );

			$html[] = '<div id="WooZoneVCSearch_iw-section-content" class="WooZoneVCSearch_iw-section-dashboard">';

			
			$html[] = 	'<div class="WooZoneVCSearch_iw-section-dashboard-row">';
			
			$html[] = 		'<div class="WooZoneVCSearch_iw-dashboard-box WooZoneVCSearch_iw-box-size-2">';
			$html[] = 			'<div class="WooZoneVCSearch_iw-dashboard-box-content">';
			$html[] = 				'<h3 class="dashboard-box-label">Product Support</h3>';
			$html[] = 				'<div class="WooZoneVCSearch_iw-dashboard-in-box-content">';
			$html[] = 					'<div class="WooZoneVCSearch_iw-row-icons">';
			$html[] = 						'<i class="fa fa-file-text-o"></i>';
			$html[] = 						'<div class="WooZoneVCSearch_iw-icons-title">';
			$html[] = 							'<p class="WooZoneVCSearch_iw-icons-strong"><a href="http://docs.aa-team.com/azon-addon/documentation/about-how-to-use/" target="_blank"/>Online Documentation</a></p>';
			$html[] = 							'<p>The best start for <strong>Azon Addon</strong> beginners</p>';
			$html[] = 						'</div>';
			$html[] = 					'</div>';
			$html[] = 					'<div class="WooZoneVCSearch_iw-row-icons">';
			$html[] = 						'<i class="fa fa-life-ring"></i>';
			$html[] = 						'<div class="WooZoneVCSearch_iw-icons-title">';
			$html[] = 							'<p class="WooZoneVCSearch_iw-icons-strong">Ticket Support</p>';
			$html[] = 							'<p>Direct help from our qualified support team</p>';
			if( !$plugin_register && trim($plugin_register) == "" ){
				$html[] = 						'<a href="#" class="WooZoneVCSearch_iw-dashboard-button WooZoneVCSearch_iw-color-red WooZoneVCSearch_iw-require_register">Unlock Now</a>';
			}else{
				$html[] = 						'<a href="http://support.aa-team.com/" target="_blank" class="WooZoneVCSearch_iw-dashboard-button WooZoneVCSearch_iw-color-yellow">Open New Ticket</a>';
			}
			$html[] = 						'</div>';
			$html[] = 					'</div>';
			$html[] = 					'<div class="WooZoneVCSearch_iw-row-icons">';
			$html[] = 						'<i class="fa fa-money"></i>';
			$html[] = 						'<div class="WooZoneVCSearch_iw-icons-title">';
			$html[] = 							'<p class="WooZoneVCSearch_iw-icons-strong">Custom Modifications</p>';
			$html[] = 							'<p>Direct help from our qualified dev and design team</p>';
			if( !$plugin_register && trim($plugin_register) == "" ){
				$html[] = 						'<a href="#" class="WooZoneVCSearch_iw-dashboard-button WooZoneVCSearch_iw-color-red WooZoneVCSearch_iw-require_register">Unlock Now</a>';
			}else{
				$html[] = 						'<a href="http://support.aa-team.com/custom-modifications/" target="_blank" class="WooZoneVCSearch_iw-dashboard-button WooZoneVCSearch_iw-color-yellow">Request a Quote</a>';
			}
			$html[] = 						'</div>';
			$html[] = 					'</div>';
			$html[] = 				'</div>';
			$html[] = 			'</div>';
			$html[] = 		'</div>';

			$current_user = wp_get_current_user();

			$html[] = 		'<div class="WooZoneVCSearch_iw-dashboard-box WooZoneVCSearch_iw-box-size-2">';
			$html[] = 			'<div class="WooZoneVCSearch_iw-dashboard-box-content">';

			if( $plugin_register && trim($plugin_register) != "" ){
				$html[] = 				'<h3 class="dashboard-box-label">Your plugin is activated</h3>';
				$html[] = 				'<div class="WooZoneVCSearch_iw-dashboard-in-box-content WooZoneVCSearch_iw-aa-register-text">' . ( $plugin_register ) . '</div>';
			}else{

				$html[] = 				'<h3 class="dashboard-box-label">Plugin Activation</h3>';
				$html[] = 				'<div class="WooZoneVCSearch_iw-dashboard-in-box-content">';
				$html[] = 					'<form class="WooZoneVCSearch_iw-register_plugin" action="" method="POST">';
				$html[] = 						'<div class="WooZoneVCSearch_iw-row-icons">';
				$html[] = 							'<i class="fa fa-key"></i>';
				$html[] = 							'<div class="WooZoneVCSearch_iw-icons-title">';
				$html[] = 								'<p class="WooZoneVCSearch_iw-icons-strong">Item Purchase Code (IPC)</p>';
				$html[] = 								'<p>You can learn how to find your purchase code <a href="http://support.aa-team.com/themes/default/assets/images/ipc-help.png" target="_blank">here</a></p>';
				$html[] = 							'</div>';
				$html[] = 						'</div>';
				$html[] = 						'<input type="text" value="" style="width:100%" class="WooZoneVCSearch_iw-dashboard-input" name="WooZoneVCSearch_iw-validation-token" placeholder="IPC Code">';
				$html[] = 						'<input type="text" value="' . ( $current_user->data->user_email ) . '" style="width:100%" class="WooZoneVCSearch_iw-dashboard-input" name="WooZoneVCSearch_iw-validation-email" placeholder="Register email">';
				$html[] = 						'<p>Reminder! The Regular License is for One Install Only!</p>';
				$html[] = 						'<input type="button" class="WooZoneVCSearch_iw-dashboard-button WooZoneVCSearch_iw-color-yellow" value="Register the code" />';
				$html[] = 					'</form>';
				$html[] = 				'</div>';
			}
			$html[] = 			'</div>';
			$html[] = 		'</div>';

			$html[] = 		'<div class="WooZoneVCSearch_iw-dashboard-box WooZoneVCSearch_iw-box-size-2">';
			$html[] = 			'<div class="WooZoneVCSearch_iw-dashboard-box-content">';
			$html[] = 				'<h3 class="dashboard-box-label">System Requirements</h3>';
			$html[] = 				'<div class="WooZoneVCSearch_iw-dashboard-in-box-content">';
			$html[] = 					'<table class="WooZoneVCSearch_iw-system-req">';
			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">' . ( is_multisite() ? 'WPMU' : 'WP' ) . ' Version:</td>';
			$html[] = 							'<td><span class="WooZoneVCSearch_iw-success"><i class="fa fa-check-square"></i></span></td>';
			$html[] = 							'<td width="30%">' . get_bloginfo('version') . ' </td>';
			$html[] = 						'</tr>';

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">Uploads folder writable</td>';
			$html[] = 							'<td><span class="WooZoneVCSearch_iw-success"><i class="fa fa-check-square"></i></span></td>';
			$html[] = 							'<td width="30%"></td>';
			$html[] = 						'</tr>';

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">Max Upload Size:</td>';
			$html[] = 							'<td><span class="WooZoneVCSearch_iw-success"><i class="fa fa-exclamation-circle"></i></span></td>';
			$html[] = 							'<td width="30%">' . ( size_format( wp_max_upload_size() ) ) . '</td>';
			$html[] = 						'</tr>';


			$allow_remote_get = (boolean) get_option( "_WooZoneVCSearch_allow_remote_get" );

			if( $allow_remote_get === false ){
				$response = wp_remote_get( 'http://google.com' );
				if( isset($response['body']) && trim($response['body']) != "" ){
					$allow_remote_get = true;
					update_option( "_WooZoneVCSearch_allow_remote_get", true );
				}
			}

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">Remote GET:</td>';
			if( $allow_remote_get ){
				$html[] = 						'<td><span class="WooZoneVCSearch_iw-success"><i class="fa fa-exclamation-circle"></i></span></td>';
			}else{
				$html[] = 						'<td><span class="WooZoneVCSearch_iw-error"><i class="fa fa-check-square"></i></span></td>';
			}
			$html[] = 							'<td width="30%"></td>';
			$html[] = 						'</tr>';
			$html[] = 						'<tr>';
			$html[] = 							'<td>Your Theme</td>';
			$template = get_option( 'template' );
			$themes = wp_get_themes();
			$theme = $themes[$template];

			//var_dump('<pre>',$theme ,'</pre>'); die; 
			$html[] = 							'<td><span class="WooZoneVCSearch_iw-success"><i class="fa fa-check-square"></i></span></td>';
			$html[] = 							'<td><a href="' . ( admin_url( '/themes.php' ) ) . '"><img width="80" src="' . ( esc_url( $theme->get_screenshot() ) ) . '" /></a></td>';
			$html[] = 						'</tr>';

			$html[] = 						'<tr>';
			$html[] = 							'<td width="55%">WP DEBUG:</td>';
			$html[] = 							'<td rowspan="2">' . (  defined('WP_DEBUG') && WP_DEBUG === true ? 'YES' : "NO" ) . '</td>';
			$html[] = 						'</tr>';

			$html[] = 					'</table>';
			$html[] = 				'</div>';
			$html[] = 			'</div>';
			$html[] = 		'</div>';

			$html[] = 	'</div>';
			$html[] = '</div>';
			
			return implode( "\n", $html );
		}
	}
}