<?php
/**
 * WooZoneVCSearch Main manager.
 *
 * @package WooZoneVCSearch
 * @since   1.0
 */
if ( ! function_exists( 'WooZoneVCSearch' ) ) {
	/**
	 * WooZoneVCSearch manager.
	 * @since 1.0
	 * @return WooZoneVCSearch
	 */
	function WooZoneVCSearch() {
		global $WooZoneVCSearch;

		return $WooZoneVCSearch;
	}
}

if ( ! function_exists( 'WooZoneVCSearch_Settings' ) ) {
	/**
	 * WooZoneVCSearch manager.
	 * @since 1.0
	 * @return WooZoneVCSearch_Settings
	 */
	function WooZoneVCSearch_Settings() {
		return new WooZoneVCSearch_Settings();
	}
}

if ( ! function_exists( 'WooZoneVCSearch_Social' ) ) {
	/**
	 * WooZoneVCSearch manager.
	 * @since 1.0
	 * @return WooZoneVCSearch_Social
	 */
	function WooZoneVCSearch_Social() {
		return new WooZoneVCSearch_Social();
	}
}

if ( ! function_exists( 'WooZoneVCSearch_Settings_Build_Options' ) ) {
	/**
	 * WooZoneVCSearch manager.
	 * @since 1.0
	 * @return WooZoneVCSearch_Settings
	 */
	function WooZoneVCSearch_Settings_Build_Options( $options=array(), $box_id='' ) {
		return WooZoneVCSearch_Settings()->build_options( $options, $box_id );
	}
}

/*
if ( ! function_exists( 'WooZoneVCSearch_Envato_Helper' ) ) {
	function WooZoneVCSearch_Envato_Helper() {
		return new WooZoneVCSearch_Envato_Helper();
	}
}*/

if ( ! function_exists( 'WooZoneVCSearch_Woocommerce' ) ) {
	/**
	 * WooZoneVCSearch manager.
	 * @since 1.0
	 * @return WooZoneVCSearch_Settings
	 */
	function WooZoneVCSearch_Woocommerce() {
		return new WooZoneVCSearch_Woocommerce();
	}
}

if ( ! function_exists( 'WooZoneVCSearch_Load_Options' ) ) {
	/**
	 * WooZoneVCSearch manager.
	 * @since 1.0
	 * @return WooZoneVCSearch_Settings
	 */
	function WooZoneVCSearch_Load_Options( $dir ) {
	    new WooZoneVCSearch_Settings(); // save settings first!
	    
		$options = array();
		// find if we have a options.php into the same folder
		$options_file = $dir . '/options.php';
		if( is_file($options_file) ){
			
			ob_start();
			require_once $options_file;
			
			$content = ob_get_contents();
			ob_clean();
 
			if( trim($content) != "" ){
				$options = json_decode( $content, true );
			}
		}
		return $options;
	}
}

if ( ! function_exists( 'WooZoneVCSearch_mapper' ) ) {
	/**
	 * Shorthand for WooZoneVCSearch Mapper.
	 * @since 1.0
	 * @return WooZoneVCSearch_Mapper
	 */
	function WooZoneVCSearch_mapper() {
		return WooZoneVCSearch()->mapper();
	}
}
if ( ! function_exists( 'WooZoneVCSearch_settings' ) ) {
	/**
	 * Shorthand for WooZoneVCSearch settings.
	 * @since 1.0
	 * @return WooZoneVCSearch_Settings
	 */
	function WooZoneVCSearch_settings() {
		return WooZoneVCSearch()->settings();
	}
}
if ( ! function_exists( 'WooZoneVCSearch_path_dir' ) ) {
	/**
	 * Get file/directory path in WooZoneVCSearch.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 1.0
	 * @return string
	 */
	function WooZoneVCSearch_path_dir( $name, $file = '' ) {
		return WooZoneVCSearch()->path( $name, $file );
	}
}
if ( ! function_exists( 'WooZoneVCSearch_upload_dir' ) ) {
	/**
	 * Temporary files upload dir;
	 * @since 1.0
	 * @return string
	 */
	function WooZoneVCSearch_upload_dir() {
		return WooZoneVCSearch()->uploadDir();
	}
}
if ( ! function_exists( 'WooZoneVCSearch_path_dir' ) ) {
	/**
	 * Get file/directory path in WooZoneVCSearch.
	 *
	 * @param string $name - path name
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function WooZoneVCSearch_path_dir( $name, $file = '' ) {
		return WooZoneVCSearch()->path( $name, $file );
	}
}
if ( ! function_exists( 'WooZoneVCSearch_post_param' ) ) {
	/**
	 * Get param value from $_POST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function WooZoneVCSearch_post_param( $param, $default = null ) {
		return isset( $_POST[ $param ] ) ? $_POST[ $param ] : $default;
	}
}
if ( ! function_exists( 'WooZoneVCSearch_get_param' ) ) {
	/**
	 * Get param value from $_GET if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 1.0
	 * @return null|string - null for undefined param.
	 */
	function WooZoneVCSearch_get_param( $param, $default = null ) {
		return isset( $_GET[ $param ] ) ? $_GET[ $param ] : $default;
	}
}
if ( ! function_exists( 'WooZoneVCSearch_request_param' ) ) {
	/**
	 * Get param value from $_REQUEST if exists.
	 *
	 * @param $param
	 * @param $default
	 *
	 * @since 4.4
	 * @return null|string - null for undefined param.
	 */
	function WooZoneVCSearch_request_param( $param, $default = null ) {
		return isset( $_REQUEST[ $param ] ) ? $_REQUEST[ $param ] : $default;
	}
}
if ( ! function_exists( 'WooZoneVCSearch_action' ) ) {
	/**
	 * Get WooZoneVCSearch special action param.
	 * @since 1.0
	 * @return string|null
	 */
	function WooZoneVCSearch_action() {
		$WooZoneVCSearch_action = 'dashboard';
		if ( isset( $_GET['WooZoneVCSearch_action'] ) ) {
			$WooZoneVCSearch_action = $_GET['WooZoneVCSearch_action'];
		} elseif ( isset( $_POST['WooZoneVCSearch_action'] ) ) {
			$WooZoneVCSearch_action = $_POST['WooZoneVCSearch_action'];
		}

		return $WooZoneVCSearch_action;
	}
}

if ( ! function_exists( 'WooZoneVCSearch_asset_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function WooZoneVCSearch_asset_url( $file ) {
		return WooZoneVCSearch()->assetUrl( $file );
	}
}

if ( ! function_exists( 'WooZoneVCSearch_templates_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function WooZoneVCSearch_templates_url( $file ) {
		return WooZoneVCSearch()->templatesUrl( $file );
	}
}

if ( ! function_exists( 'WooZoneVCSearch_shortcodes_url' ) ) {
	/**
	 * Get full url for assets.
	 *
	 * @param string $file
	 *
	 * @since 4.2
	 * @return string
	 */
	function WooZoneVCSearch_shortcodes_url( $file ) {
		return WooZoneVCSearch()->shortcodesUrl( $file );
	}
}

if ( ! function_exists( 'WooZoneVCSearch_locate_template' ) ) {
	function WooZoneVCSearch_locate_template( $template_name, $template_path = '', $default_path = '' ) 
	{
	    if ( ! $template_path ) {
	        $template_path = 'templates/';
	    }

	    if ( ! $default_path ) {
	        $default_path = WooZoneVCSearch()->path( 'TEMPLATES_DIR', '' );
	    }
 
	    // Look within passed path within the theme - this is priority.
	    $template = locate_template(
	        array(
	            trailingslashit( $template_path ) . $template_name,
	            $template_name
	        )
	    );

	    // Get default template/
	    if ( ! $template ) {
	        $template = $default_path . $template_name;
	    }

	    // Return what we found.
	    return apply_filters( 'WooZoneVCSearch_locate_template', $template, $template_name, $template_path );
	}
}


if ( ! function_exists( 'WooZoneVCSearch_Get_IP' ) ) {
	function WooZoneVCSearch_Get_IP() 
	{
		//Just get the headers if we can or else use the SERVER global
		if ( function_exists( 'apache_request_headers' ) ) {
			$headers = apache_request_headers();
		} else {
			$headers = $_SERVER;
		}
		//Get the forwarded IP if it exists
		if ( array_key_exists( 'X-Forwarded-For', $headers ) && filter_var( $headers['X-Forwarded-For'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
			$the_ip = $headers['X-Forwarded-For'];
		} elseif ( array_key_exists( 'HTTP_X_FORWARDED_FOR', $headers ) && filter_var( $headers['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 )
		) {
			$the_ip = $headers['HTTP_X_FORWARDED_FOR'];
		} else {
			$the_ip = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );
		}
		return $the_ip;
	}
}

/**
 * Plugin name for WooZoneVCSearch_time_elapsed_string.
 *
 * @since 1.0
 * @return string
 */
if ( ! function_exists( 'WooZoneVCSearch_time_elapsed_string' ) ) {
	function WooZoneVCSearch_time_elapsed_string($ptime)
	{
	    $etime = time() - $ptime;
	
	    if ($etime < 1)
	    {
	        return '0 seconds';
	    }
	
	    $a = array( 365 * 24 * 60 * 60  =>  'year',
	                 30 * 24 * 60 * 60  =>  'month',
	                      24 * 60 * 60  =>  'day',
	                           60 * 60  =>  'hour',
	                                60  =>  'minute',
	                                 1  =>  'second'
	                );
	    $a_plural = array( 'year'   => 'years',
	                       'month'  => 'months',
	                       'day'    => 'days',
	                       'hour'   => 'hours',
	                       'minute' => 'minutes',
	                       'second' => 'seconds'
	                );
	
	    foreach ($a as $secs => $str)
	    {
	        $d = $etime / $secs;
	        if ($d >= 1)
	        {
	            $r = round($d);
	            return $r . ' ' . ($r > 1 ? $a_plural[$str] : $str) . ' ago';
	        }
	    }
	}
}


/**
 * Plugin name for WooZoneVCSearch.
 *
 * @since 1.0
 * @return string
 */
function WooZoneVCSearch_plugin_name() {
	return WooZoneVCSearch()->pluginName();
}