<?php 
class WooZoneVCSearchShortcodeCustomList {

	public $the_plugin = null;
	public $shortcode_alias = '';

	public function __construct() 
	{
		$this->the_plugin = $GLOBALS['WooZoneVCSearch'];

		$this->shortcode_alias = $this->the_plugin->alias . 'custom_list';

		// We safely integrate with VC with this hook
		add_action( 'vc_after_init', array( $this, 'vc_init' ) );

		add_shortcode( $this->shortcode_alias, array( $this, 'shortcode_output' ) );
	}

	public function vc_init()
	{
		// Check if Visual Composer is installed
		if ( ! defined( 'WPB_VC_VERSION' ) ) {
			// Display notice that Visual Compser is required
			add_action('admin_notices', array( $this, 'showVcVersionNotice' ));
			return;
		}

		/*
		Add your Visual Composer logic here.
		Lets call vc_map function to "register" our custom shortcode within Visual Composer interface.

		More info: https://wpbakery.atlassian.net/wiki/spaces/VC/pages/524332/vc+map

		*/
		vc_map(array(
			"name" => __("Azon - Custom ASIN list", "my-text-domain"),
			"base" => $this->shortcode_alias, 
			"class" => "",
			"description" => 'Add a custom Amazon ASIN list.',
			"category" => __("Azon Addons", "my-text-domain"),
			'admin_enqueue_js' => WooZoneVCSearch_asset_url( 'vc_extend.js' ),
			"icon" => WooZoneVCSearch_asset_url( 'icons/list.png' ),
			"admin_label" => false,
			"params" => array(

				array(
					'type' => $this->the_plugin->alias . '_country_dropdown',
					'heading' => __( 'Search in the following Amazon Website:', 'js_composer' ),
					'param_name' => 'search_country',
					'value' => "com",
				),

				array(
					'type' => 'textarea',
					'heading' => __( 'Amazon ASIN codes:', 'js_composer' ),
					'param_name' => 'asins',
					'description' => __( 'Simply copy/paste the Amazon ASIN list here. Separated by comma.', 'js_composer' )
				),

				
				array(
					'type' => 'dropdown',
					'heading' => __( 'Display pagination:', 'js_composer' ),
					'param_name' => 'show_pagination',
					'std' => 'yes',
					'save_always' => true,
					'value' => array(
						'YES' => 'yes',
						'NO' => 'no'
					),
					'description' => __( 'Set if you want to display pagination on products', 'js_composer' ),
				),

				array(
					'type' => 'dropdown',
					'heading' => __( 'Products per page:', 'js_composer' ),
					'param_name' => 'items_per_page',
					'std' => 4,
					'value' => range(1, 20),
					'description' => __( 'Select the number of products per page.', 'js_composer' ),
				),/**/

				array(
					'type' => $this->the_plugin->alias . '_html_radio_settings',
					'heading' => __( 'Products per Column:', 'js_composer' ),
					'param_name' => 'products_per_column',
					'value' => '',
					'description' => __( 'Select how many products to display per column', 'js_composer' ),
					'options'           => array(
						'1' => array(
							'caption' => '1 item / column',
							'html' => '<div class="WooZoneVCSearch-grid-size" data-size="1">
								<i></i>
							</div>'
						),

						'2' => array(
							'caption' => '2 items / column',
							'html' => '<div class="WooZoneVCSearch-grid-size" data-size="2">
								<i></i>
								<i></i>
							</div>'
						),

						'3' => array(
							'caption' => '3 items / column',
							'html' => '<div class="WooZoneVCSearch-grid-size" data-size="3">
								<i></i>
								<i></i>
								<i></i>
							</div>'
						),

						'4' => array(
							'caption' => '4 items / column',
							'html' => '<div class="WooZoneVCSearch-grid-size" data-size="4">
								<i></i>
								<i></i>
								<i></i>
								<i></i>
							</div>'
						),

						'5' => array(
							'caption' => '5 items / column',
							'html' => '<div class="WooZoneVCSearch-grid-size" data-size="5">
								<i></i>
								<i></i>
								<i></i>
								<i></i>
								<i></i>
							</div>'
						),

						'6' => array(
							'caption' => '6 items / column',
							'html' => '<div class="WooZoneVCSearch-grid-size" data-size="6">
								<i></i>
								<i></i>
								<i></i>
								<i></i>
								<i></i>
								<i></i>
							</div>'
						),
					)
				),

				array(
					"type" => $this->the_plugin->alias . '_typo_settings',
					"holder" => "div",
					"class" => "",
					'save_always' => true,
					'options'           => array(

						'product_title' => array(
							'title' => 'Product Title',
							'size' => '14px',
							'weight' => 'bold',
							'color' => '#131313',
							'family' => 'Droid Sans',
						),

						'product_sub_title' => array(
							'title' => 'Product Sub Title',
							'size' => '14px',
							'weight' => 'bold',
							'color' => '#131313',
							'family' => 'Droid Sans',
						),

						'nb_of_reviews' => array(
							'title' => 'Number of reviews',
							'size' => '12px',
							'weight' => 'normal',
							'color' => '#ff7209',
							'family' => 'Droid Sans',
						),

						'buy_btn' => array(
							'title' => 'Buy button',
							'size' => '12px',
							'color' => '#fff',
							'family' => 'Droid Sans',
							'background-color' => '#ff7209'
						),

						'regular_price' => array(
							'title' => 'Regular price text',
							'size' => '14px',
							'color' => '#afafaf',
							'family' => 'Droid Sans',
						),

						'sale_price' => array(
							'title' => 'Sale price text',
							'size' => '20px',
							'color' => '#121212',
							'family' => 'Droid Sans',
						),
						
						'pagination_color' => array(
							'title' => 'Pagination Color - Active State',
							'size' => '20px',
							'color' => '#121212',
							'family' => 'Droid Sans',
						),

						'pagination_default' => array(
							'title' => 'Pagination Color - Default Color ',
							'size' => '20px',
							'color' => '#121212',
							'family' => 'Droid Sans',
						),
					),
					"heading" => __("Typography", "my-text-domain"),
					"param_name" => "typo",
					"value" => __("", "my-text-domain"),
					"description" => __("Typography", "my-text-domain")
				)
			)
		));
	}

	public function custom_frontend_style( $ret_style_id='', $style_id='' )
	{
		if( isset( $style_id ) && $style_id != '' ) {
			$custom_uniqid = $style_id;
		} else {
			$custom_uniqid = uniqid();
		}
		$settings = $this->the_plugin->settings['setup'];
		$css = array();

		//echo json_encode( $settings ) ;

		// custom cases here ...
		if( isset($settings['products_per_column']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . " .WooZoneVCSearch-amazon-response  {
					grid-template-columns: repeat(" . ( $settings['products_per_column'] ) . ", minmax(0, 1fr));
				}
			";
		}

		if( isset($settings['template_typography']['headline']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . " .WooZoneVCSearch-box-title {
					font-family: '" . ( $settings['template_typography']['headline']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['headline']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['headline']['weight'] ) . ";
					color: " . ( $settings['template_typography']['headline']['color'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['product_title']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . ".WooZoneVCSearch-amazon-products .WooZoneVCSearch-product .WooZoneVCSearch-title a {
					font-family: '" . ( $settings['template_typography']['product_title']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['product_title']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['product_title']['weight'] ) . ";
					color: " . ( $settings['template_typography']['product_title']['color'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['product_sub_title']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . ".WooZoneVCSearch-amazon-products .WooZoneVCSearch-product .WooZoneVCSearch-subtitle {
					font-family: '" . ( $settings['template_typography']['product_sub_title']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['product_sub_title']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['product_sub_title']['weight'] ) . ";
					color: " . ( $settings['template_typography']['product_sub_title']['color'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['nb_of_reviews']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . ".WooZoneVCSearch-amazon-products .WooZoneVCSearch-product .WooZoneVCSearch-rating-totals {
					font-family: '" . ( $settings['template_typography']['nb_of_reviews']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['nb_of_reviews']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['nb_of_reviews']['weight'] ) . ";
					color: " . ( $settings['template_typography']['nb_of_reviews']['color'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['regular_price']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . ".WooZoneVCSearch-amazon-products .WooZoneVCSearch-product .WooZoneVCSearch-price div {
					font-family: '" . ( $settings['template_typography']['regular_price']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['regular_price']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['regular_price']['weight'] ) . ";
					color: " . ( $settings['template_typography']['regular_price']['color'] ) . ";
				}
			";
		}


		if( isset($settings['template_typography']['sale_price']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . ".WooZoneVCSearch-amazon-products .WooZoneVCSearch-product .WooZoneVCSearch-price .WooZoneVCSearch-new-price {
					font-family: '" . ( $settings['template_typography']['sale_price']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['sale_price']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['sale_price']['weight'] ) . ";
					color: " . ( $settings['template_typography']['sale_price']['color'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['buy_btn']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . ".WooZoneVCSearch-amazon-products .WooZoneVCSearch-product .WooZoneVCSearch-button {
					font-family: '" . ( $settings['template_typography']['buy_btn']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['buy_btn']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['buy_btn']['weight'] ) . ";
					color: " . ( $settings['template_typography']['buy_btn']['color'] ) . ";
					background-color: " . ( $settings['template_typography']['buy_btn']['background'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['pagination_default']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . " .WooZoneVCSearch-custom-list-pagination a, 
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . " .WooZoneVCSearch-feed-pagination a {
					font-family: '" . ( $settings['template_typography']['pagination_default']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['pagination_default']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['pagination_default']['weight'] ) . ";
					color: " . ( $settings['template_typography']['pagination_default']['color'] ) . ";
				}
			";
		}

		if( isset($settings['template_typography']['pagination_color']) ){
			$css[] = "
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . " .WooZoneVCSearch-custom-list-pagination a.is_on, 
				body .WooZoneVCSearch-uniq-" . ( $custom_uniqid ) . " .WooZoneVCSearch-feed-pagination a.is_on {
					font-family: '" . ( $settings['template_typography']['pagination_color']['family'] ) . "';
					font-size: " . ( $settings['template_typography']['pagination_color']['size'] ) . "px;
					font-weight: " . ( $settings['template_typography']['pagination_color']['weight'] ) . ";
					color: " . ( $settings['template_typography']['pagination_color']['color'] ) . ";
					border-color: " . ( $settings['template_typography']['pagination_color']['color'] ) . ";
				}
			";
		}

		$buffer = implode( "\n", $css );

		// Remove comments
		$buffer = preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $buffer);
		 
		// Remove space after colons
		$buffer = str_replace(': ', ':', $buffer);
		 
		// Remove whitespace
		$buffer = str_replace(array("\r\n", "\r", "\n", "\t", '    ', '    '), '', $buffer);

		if( isset( $ret_style_id ) && $ret_style_id == 'ret_style_id' ) {
			return $custom_uniqid;
		} else {
			return $buffer;
		}
	}

	public function add_styles( $style_id = '' )
	{
		$inline_styles = $this->custom_frontend_style( '', $style_id ); 
		wp_enqueue_style( WooZoneVCSearch()->alias . '-Droid+Sans', 'https://fonts.googleapis.com/css?family=Droid+Sans:400,700', array(), '1.0' );
		wp_enqueue_style( WooZoneVCSearch()->alias . '-amazon-styles', WooZoneVCSearch()->assetUrl('frontend.css'), array(), '1.0' );
		wp_add_inline_style( WooZoneVCSearch()->alias . '-amazon-styles', $inline_styles);
		wp_enqueue_style( WooZoneVCSearch()->alias . '-rating', WooZoneVCSearch()->assetUrl('star-rating-svg.css'), array(), '1.0' );

		wp_enqueue_script( WooZoneVCSearch()->alias . '-amazon-rating', WooZoneVCSearch()->assetUrl('jquery.star-rating-svg.js'), array( 'jquery' ) );
		wp_enqueue_script( WooZoneVCSearch()->alias . '-amazon', WooZoneVCSearch()->assetUrl('frontend.js'), array( 'jquery' ) );

		wp_localize_script( WooZoneVCSearch()->alias . '-amazon', WooZoneVCSearch()->alias . '_custom', array(
			'ajaxurl' => admin_url('admin-ajax.php'),
			'translation' => array(
				'no_results_found' => esc_html__('No more results found..', 'WooZoneVCSearch'),
				'nb_results' => esc_html__('Around {nb_results} results found..', 'WooZoneVCSearch'),
				'prev_page' => esc_html__('Previous page', 'WooZoneVCSearch'),
				'next_page' => esc_html__('Next page', 'WooZoneVCSearch')
			),
		));

		if( count($this->the_plugin->settings['setup']['template_typography']) > 0 ){
			$google_fonts = array();
			foreach ($this->the_plugin->settings['setup']['template_typography'] as $key => $value) {
				$google_fonts[] = str_replace( " ", "+", $value['family']);
			}

			$google_fonts = array_unique( $google_fonts );

			wp_enqueue_style( WooZoneVCSearch()->alias . '-googlefonts', '//fonts.googleapis.com/css?family=' . implode( "|", $google_fonts ), array(), '1.0' );
		}
	}

	public function shortcode_output( $atts )
	{
		extract( shortcode_atts( array(
			'show_pagination' => 'no',
			'items_per_page' => 2,
			'asins' => '',
			'products_per_column' => 3,
			'search_category' => 'All',
			'search_country' => 'com',
			'typo' => "{'product_title':{'family':'Droid Sans','size':'14','weight':'bold','color':'#131313'},'product_sub_title':{'family':'Droid Sans','size':'14','weight':'bold','color':'#131313'},'nb_of_reviews':{'family':'Droid Sans','size':'12','weight':'normal','color':'#ff7209'},'buy_btn':{'family':'Droid Sans','size':'12','weight':'normal','color':'#fff','background':'#ff7209'},'regular_price':{'family':'Droid Sans','size':'14','weight':'normal','color':'#afafaf'},'sale_price':{'family':'Droid Sans','size':'20','weight':'normal','color':'#121212'}}",
		), $atts ) );

		if( trim($typo) != "" ){
			$typo = str_replace( "'", '"', $typo );
			$typo = json_decode( $typo, true );
			$this->the_plugin->settings['setup']['template_typography'] = $typo;
		}

		$this->the_plugin->settings['setup']['items_per_page'] = $items_per_page;
		$this->the_plugin->settings['setup']['show_pagination'] = $show_pagination;
		$this->the_plugin->settings['setup']['products_per_column'] = $products_per_column;
		$this->the_plugin->settings['setup']['search_country'] = $search_country;
		$this->the_plugin->settings['setup']["geo_targeting"]['default_aff'] = $search_country;
		$this->the_plugin->settings['setup']['searchindex'] = $search_category;

		$style_id = $this->custom_frontend_style( 'ret_style_id' );
		$this->add_styles( $style_id );

		$user_country = $this->the_plugin->get_country_perip_external();

		// force for testing request 
		$user_country = isset($_GET['force_country']) && strlen($_GET['force_country']) < 5 ? $_GET['force_country'] : $user_country;

		$amz_sites = $this->the_plugin->amazon_sites();
		$amz_site = $amz_sites[$this->the_plugin->settings['setup']['search_country']];
		$amzUser = $this->the_plugin->amzForUser( $user_country, $this->the_plugin->settings['setup']['search_country'] );

		$html = array();
		$html[] = '<div class="' . ( WooZoneVCSearch()->alias ) . '-amazon-products WooZoneVCSearch-uniq-' . ( $style_id ) . '" data-type="custom-list" data-country="' . ( $user_country ) . '" data-boxid="' . ( $style_id ) . '" data-amzsite="' . ( $this->the_plugin->settings['setup']['search_country'] ) . '" data-apiurl="' . ( $amz_site['api_url'] ) . '" data-placeholder="' . ( WooZoneVCSearch()->assetUrl('images/placeholder.png') ) . '" data-perpage="' . ( $this->the_plugin->settings['setup']['items_per_page'] ) . '" data-asins="' . ( $asins ) . '" data-btntext="' . ( $this->the_plugin->settings['setup']['button_text'] ) . '" data-buyurl="' . ($this->the_plugin->buy_url( $user_country, $amzUser ) ) . '" data-searchindex="' . ( $this->the_plugin->settings['setup']['searchindex'] ) . '" data-column="' . ( $products_per_column ) . '" data-showpagination="' . ( $this->the_plugin->settings['setup']['show_pagination'] ) . '">';


		$html[] =   '<div class="' . ( WooZoneVCSearch()->alias ) . '-amazon-response"></div>';
		$html[] =   '<div class="' . ( WooZoneVCSearch()->alias ) . '-amazon-pag-response"></div>';
		$html[] = '</div>';

		return implode( "\n", $html );
	}
}

new WooZoneVCSearchShortcodeCustomList();